#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

import os
import re
import tempfile
import getpass

from base64 import b64encode
from datetime import datetime
from urllib2 import urlopen, Request, HTTPError
from urlparse import urlsplit

from xml.etree import ElementTree
from htmlentitydefs import name2codepoint

from email.utils import parsedate_tz, mktime_tz
from os.path import join, dirname, isfile

from acrylamid import log
from acrylamid.utils import FileEntry, event, escape, system
from acrylamid.errors import AcrylamidException

# no joke
USED_WORDPRESS = False


class InvalidSource(Exception):
    pass


def unescape(s):
    return re.sub('&(%s);' % '|'.join(name2codepoint),
            lambda m: unichr(name2codepoint[m.group(1)]), s)


def convert(data, fmt='markdown'):
    """Reconversion of HTML to Markdown or reStructuredText.  Defaults to Markdown,
    but can be in fact every format pandoc supports. If pandoc is not available, try
    some specific conversion tools like html2text and html2rest.

    :param html: raw content to convert to
    :param html: format to reconvert to"""

    if fmt in ('Markdown', 'markdown', 'mkdown', 'md', 'mkd'):
        cmds = ['html2text']
        fmt = 'markdown'
    elif fmt in ('rst', 'restructuredtext', 'rest', 'reStructuredText'):
        cmds = ['html2rest']
        fmt = 'rst'
    else:
        cmds = []

    cmds.insert(0, ['pandoc', '--normalize', '-f', 'html', '-t', fmt, '--strict'])

    if fmt == 'html':
        return data, 'html'

    #  - item.find(foo).text returns None if no CDATA
    #  - pandoc waits for input if a zero-length string is given
    if data is None or data is '':
        return '', fmt

    for cmd in cmds:
        try:
            return system(cmd, stdin=str(data.decode('utf-8'))), fmt.lower()
        except AcrylamidException as e:
            log.warn(e.message)
        except OSError:
            pass
    else:
        return data, 'html'


def rss20(xml):

    def parse_date_time(stamp):
        ts = parsedate_tz(stamp)
        ts = mktime_tz(ts)
        return datetime.fromtimestamp(ts)

    try:
        tree = ElementTree.fromstring(xml)
    except ElementTree.ParseError:
        raise InvalidSource('no well-formed XML')
    if tree.tag != 'rss' or tree.attrib.get('version') != '2.0':
        raise InvalidSource('no RSS 2.0 feed')

    # --- site settings --- #
    defaults = {}
    channel = tree.getchildren()[0]

    for k, v in {'title': 'sitename', 'link': 'www_root', 'language': 'lang'}.iteritems():
        try:
            defaults[v] = channel.find(k).text
        except AttributeError:
            pass

    yield defaults

    # --- individual posts --- #
    for item in channel.findall('item'):

        entry = {}
        for k, v in {'title': 'title', 'date': 'pubDate', 'link': 'link',
                     'content': 'description'}.iteritems():
            try:
                entry[k] = item.find(v).text if k != 'content' \
                                             else unescape(item.find(v).text)
            except (AttributeError, TypeError):
                pass

        if filter(lambda k: not k in entry, ['title', 'date', 'link', 'content']):
            raise AcrylamidException('invalid RSS 2.0 feed: provide at least title, ' \
                                     + 'link, content and pubDate!')

        yield {'title': entry['title'],
               'content': entry['content'],
               'date': parse_date_time(entry['date']),
               'link': entry['link']}


def atom(xml):

    def parse_date_time(stamp):
        ts = parsedate_tz(stamp)
        ts = mktime_tz(ts)
        return datetime.fromtimestamp(ts)

    try:
        tree = ElementTree.fromstring(xml)
    except ElementTree.ParseError:
        raise InvalidSource('no well-formed XML')


    if not tree.tag.endswith('/2005/Atom}feed'):
        raise InvalidSource('no Atom feed')

    # --- site settings --- #
    ns = '{http://www.w3.org/2005/Atom}'  # etree Y U have stupid namespace handling?
    defaults = {}

    defaults['title'] = tree.find(ns + 'title').text
    defaults['www_root'] = tree.find(ns + 'id').text
    defaults['author'] = tree.find(ns + 'author').find(ns + 'name').text

    yield defaults

    # --- individual posts --- #
    for item in tree.findall(ns + 'entry'):
        entry = {}

        try:
            entry['title'] = item.find(ns + 'title').text
            entry['date'] = item.find(ns + 'updated').text
            entry['link'] = item.find(ns + 'link').text
            entry['content'] = item.find(ns + 'content').text
        except (AttributeError, TypeError):
            pass

        if item.find(ns + 'content').get('type', 'text') == 'html':
            entry['content'] = unescape(entry['content'])

        if filter(lambda k: not k in entry, ['title', 'date', 'link', 'content']):
            raise AcrylamidException('invalid Atom feed: provide at least title, '
                                     + 'link, content and updated!')

        yield {'title': entry['title'],
               'content': entry['content'],
               'date': datetime.strptime(entry['date'], "%Y-%m-%dT%H:%M:%SZ"),
               'link': entry['link']}


def wp(xml):
    """WordPress to Acrylamid, stolen from pelican-import.py, thank you Alexis.
    -- https://github.com/ametaireau/pelican/blob/master/pelican/tools/pelican_import.py
    """

    global USED_WORDPRESS
    USED_WORDPRESS = True

    try:
        from BeautifulSoup import BeautifulStoneSoup
    except ImportError:
        raise AcrylamidException('BeautifulSoup is required for WordPress import')

    soup = BeautifulStoneSoup(xml)
    items = soup.rss.channel.findAll('item')

    # --- site settings --- #
    title = soup.rss.channel.fetch('title')[0].contents[0]
    www_root = soup.rss.channel.fetch('link')[0].contents[0]

    yield {'title': title, 'www_root': www_root}

    # --- individual posts --- #
    for item in items:
        if item.fetch('wp:status')[0].contents[0] == "publish":

            title = item.title.contents[0]
            link = item.fetch('link')[0].contents[0]

            content = item.fetch('content:encoded')[0].contents[0]
            content = content.replace('\n', '<br />\n')

            raw_date = item.fetch('wp:post_date')[0].contents[0]
            date = datetime.strptime(raw_date, "%Y-%m-%d %H:%M:%S")

            author = item.fetch('dc:creator')[0].contents[0].title()
            tags = [tag.contents[0] for tag in item.fetch(domain='post_tag')]

            yield {'title': title, 'content': content, 'date': date, 'author': author,
                   'tags': tags, 'link': link}


def fetch(url, auth=None):
    """Fetch URL, optional with HTTP Basic Authentication."""

    if not (url.startswith('http://') or url.startswith('https://')):
        try:
            with open(url, 'r') as fp:
                content = fp.read()
            return content
        except OSError as e:
            raise AcrylamidException(e.message)

    req = Request(url)
    if auth:
        req.add_header('Authorization', 'Basic ' + b64encode(auth))

    try:
        r = urlopen(req)
    except HTTPError as e:
        raise AcrylamidException(e.msg)

    if r.getcode() == 401:
        user = raw_input('Username: ')
        passwd = getpass.getpass()
        fetch(url, user + ':' + passwd)
    elif r.getcode() == 200:
        return r.read()

    raise AcrylamidException('invalid status code %i, aborting.' % r.getcode())


def parse(content):

    failed = []
    for method in (wp, rss20, atom):
        try:
            res =  method(content)
            return res.next(), res
        # except ValueError as e:
        #     raise AcrylamidException(e.message)
        except InvalidSource as e:
            failed.append(str(e))
    else:
        raise AcrylamidException('unable to parse source, %s' % '; '.join(failed))


def build(conf, env, defaults, items, fmt, keep=False):

    def create(defaults, title, date, author, content, fmt, permalink=None, tags=None):

        global USED_WORDPRESS

        fd, tmp = tempfile.mkstemp(suffix='.txt')
        title = escape(title)

        with os.fdopen(fd, 'wb') as f:
            f.write('---\n')
            f.write('title: %s\n' % title)
            if author != defaults.get('author', None):
                f.write('author: %s\n' % author)
            f.write('date: %s\n' % date.strftime(conf['date_format']))
            f.write('filter: [%s, ]\n' % fmt)
            if tags:
                f.write('tags: [%s]\n' % ', '.join(tags))
            if permalink:
                f.write('permalink: %s\n' % permalink)
            f.write('---\n\n')

            # this are fixes for WordPress because they don't save HTML but a
            # stupid mixed-in form of HTML making it very difficult to get either HTML
            # or reStructuredText/Markdown
            if USED_WORDPRESS and fmt == 'markdown':
                content = content.replace("\n ", "  \n")
                content = content.replace("\n", "  \n")
            elif USED_WORDPRESS and fmt == 'rst':
                content = content.replace('\n ', '\n\n')
            f.write(content+'\n')

        entry = FileEntry(tmp, conf)
        p = join(conf['content_dir'], dirname(entry.permalink)[1:])

        try:
            os.makedirs(p.rsplit('/', 1)[0])
        except OSError:
            pass

        filepath = p + '.txt'
        if isfile(filepath):
            raise AcrylamidException('Entry already exists %r' % filepath)
        os.rename(tmp, filepath)
        event.create(filepath)

    for item in items:

        if keep:
            m = urlsplit(item['link'])
            permalink = m.path if m.path != '/' else None

        content, fmt = convert(item.get('content', ''), fmt)
        create(defaults, item['title'], item['date'], item['author'], content, fmt,
               tags=item.get('tags', None), permalink=permalink if keep else None)

    print "\nImport was successful. Edit your conf.py with these new settings:"
    for key, value in defaults.iteritems():
        print "    %s = '%s'" % (key.upper(), value)


__all__ = ['fetch', 'parse', 'build']



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

import os
import markdown

from acrylamid.filters import Filter
from acrylamid.errors import AcrylamidException


class Markdown(Filter):

    match = ['md', 'mkdown', 'markdown', 'Markdown']
    version = '1.0.0'

    conflicts = ['rst', 'plain']
    priority = 70.0

    extensions = dict((x, x) for x in ['abbr', 'fenced_code', 'footnotes',
                                       'headerid', 'tables', 'codehilite'])

    def init(self, conf, env):

        self.failed = []
        self.ignore = env.options.ignore

        # -- discover markdown extensions --
        for mem in os.listdir(os.path.dirname(__file__)):
            if mem.startswith('mdx_') and mem.endswith('.py'):
                try:
                    mod = __import__(mem.replace('.py', ''))
                    mdx = mod.makeExtension()
                    if isinstance(mod.match, basestring):
                        self.match.append(mod.match)
                        self.extensions[mod.__name__] = mdx
                    else:
                        for name in mod.match:
                            self.extensions[name] = mdx
                except (ImportError, Exception), e:
                    self.failed.append('%r %s: %s' % (mem, e.__class__.__name__, e))

    def __contains__(self, key):
        return True if key in self.extensions else False

    def transform(self, text, entry, *filters):

        val = []
        for f in filters:
            if f in self:
                val.append(f)
            else:
                x = f.split('(', 1)[:1][0]
                if x in self:
                    val.append(x)
                    self.extensions[x] = f
                elif not self.ignore:
                    raise AcrylamidException('Markdown: %s' % '\n'.join(self.failed))

        md = markdown.Markdown(extensions=[self.extensions[m] for m in val])
        return md.convert(text)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

import os

from acrylamid.filters import Filter
from acrylamid.utils import cached_property


class Restructuredtext(Filter):

    match = ['restructuredtext', 'rst', 'rest', 'reST', 'reStructuredText']
    version = '1.0.0'

    conflicts = ['markdown', 'plain']
    priority = 70.00

    def init(self, conf, env):

        self.failed = []
        self.extensions = {}
        self.ignore = env.options.ignore

    def transform(self, content, entry, *filters):

        self.filters = filters

        settings = {
            'initial_header_level': 1,
            'doctitle_xform': 0
        }

        parts = self.publish_parts(content, writer_name='html', settings_overrides=settings)
        return parts['body'].encode('utf-8')

    @cached_property
    def publish_parts(self):
        """On-demand import.  Importing reStructuredText and Pygments takes
        3/5 of overall runtime just to import."""

        from docutils.core import publish_parts
        from docutils.parsers.rst import directives

        # -- discover reStructuredText extensions --
        for mem in os.listdir(os.path.dirname(__file__)):
            if mem.startswith('rstx_') and mem.endswith('.py'):
                try:
                    mod = __import__(mem.replace('.py', ''))
                    rstx = mod.makeExtension()
                    if isinstance(mod.match, basestring):
                        self.match.append(mod.match)
                        self.extensions[mod.__name__] = rstx
                    else:
                        for name in mod.match:
                            self.extensions[name] = rstx
                except (ImportError, Exception), e:
                    self.failed.append('%r %s: %s' % (mem, e.__class__.__name__, e))

        for directive, klass in self.extensions.iteritems():
            directives.register_directive(directive, klass)

        return publish_parts



# License: BSD Style, 2 clauses. see acrylamid/__init__.py
# -*- encoding: utf-8 -*-

from docutils import nodes
from docutils.parsers.rst import Directive, directives

match = ['youtube', 'yt']


def align(argument):
    return directives.choice(argument, ('left', 'center', 'right'))


class YouTube(Directive):
    """reStructuredText directive that creates an embed object to display
    a video from Youtube

    Usage example::

        .. youtube:: ZPJlyRv_IGI
           :align: center
           :height: 1280
           :width: 720
    """

    required_arguments = 1
    optional_arguments = 0
    option_spec = {
        'height': directives.length_or_unitless,
        'width': directives.length_or_percentage_or_unitless,
        'border': directives.length_or_unitless,
        'align': align,
    }
    has_content = False

    def run(self):

        alignments = {
            'left': '0',
            'center': '0 auto',
            'right': '0 0 0 auto',
        }

        self.options['uri'] = 'https://www.youtube-nocookie.com/embed/' \
            + self.arguments[0]
        self.options.setdefault('width', '680')
        self.options.setdefault('height', '382px')
        self.options['align'] = alignments[self.options.get('align', 'center')]
        self.options.setdefault('border', '0')

        YT_EMBED = """<iframe width="%(width)s" height="%(height)s" src="%(uri)s" \
                      frameborder="%(border)s" style="display: block; margin: %(align)s;" \
                      class="video" allowfullscreen></iframe>"""
        return [nodes.raw('', YT_EMBED % self.options, format='html')]

def makeExtension():
    return YouTube



# Copyright (c) 2010, 2011 Will Kahn-Greene
#
# Pyblosxom is distributed under the MIT license.  See the file
# LICENSE for distribution details.
#
# -- http://pyblosxom.bluesock.org/
#
# This is a port of PyBlosxom's acronyms plugin as acrylamid
# filter. All credits go to Pyblosxom's and blosxom's authors.

import re

from acrylamid import log
from acrylamid.filters import Filter


class Acronyms(Filter):

    match = [re.compile('^Acronyms?$', re.I), 'abbr', 'Abbr']
    version = '1.0.0'

    # after Typography, so CAPS is around ABBR
    priority = 20.0

    def init(self, conf, env):

        if conf.get('acronyms_file', None):
            with open(conf['acronyms_file'], 'r') as fp:
                data = fp.readlines()
        else:
            global ACRONYMS
            data = ACRONYMS.split('\n')

        acronyms = []
        for line in data:
            line = line.split("=", 1)
            firstpart = line[0].strip()

            try:
                firstpartre = re.compile("(\\b" + firstpart + "\\b)")
            except re.error:
                log.warn("acronyms: '%s' is not a regular expression")
                continue

            secondpart = line[1].strip()
            secondpart = secondpart.replace("\"", "&quot;")

            if (secondpart.startswith("abbr|") or firstpart.endswith(".")):
                if secondpart.startswith("abbr|"):
                    secondpart = secondpart[5:]
                repl = "<abbr title=\"%s\">\\1</abbr>" % secondpart
            else:
                if secondpart.startswith("acronym|"):
                    secondpart = secondpart[8:]
                repl = "<abbr title=\"%s\">\\1</abbr>" % secondpart

            acronyms.append((firstpartre, repl))

        self.acronyms = acronyms
        self.tag_re = re.compile("<\D.*?>")
        self.tag_digit_re = re.compile("<\d+?>")

    def transform(self, text, entry, *args):

        acrolist = self.acronyms
        if len(args) > 0:
            acrolist = filter(lambda k: any(k[0].match(v) for v in args), acrolist)

        tags = {}

        def matchrepl(matchobj):
            ret = "<%d>" % len(tags)
            tags[ret] = matchobj.group(0)
            return ret

        text = self.tag_re.sub(matchrepl, text)

        for reob, repl in acrolist:
            text = reob.sub(repl, text)

        def matchrepl(matchobj):
            return tags[matchobj.group(0)]

        text = self.tag_digit_re.sub(matchrepl, text)

        return text


ACRONYMS = r"""
ASCII=American Standard Code for Information Interchange
BGP=Border Gateway Protocol
BSD=Berkeley System Distribution
CGI=Common Gateway Interface
CLI=Command Line Interface
CLUE=Command Line User Environment
CSS=Cascading Stylesheets
CVS=Concurrent Versioning System
DSL=Digital Subscriber Line
EFF=Electronic Frontier Foundation
ELF=Executable and Linking Format
FAQ=Frequently asked question(s)
FFII=Foundation for a Free Information Infrastructure / F&ouml;rderverein f&uuml;r eine Freie Informationelle Infrastruktur
FIFO=First In, First Out
FLOSS=Free, Libre and Open Source Software
FOSS=Free and Open Source Software
FSF=Free Software Foundation
FSFE=Free Software Foundation Europe
GNOME=GNU Network Object Model Environment
GPL=GNU General Public License
GPRS=General Packet Radio Service
GSM=Global System for Mobile Communications
GUI=Graphical User Interface
HTML=Hypertext Markup Language
HTTP=Hypertext Transport Protocol
IMD[Bb]=Internet Movie Database
IRC=Internet Relay Chat
ISBN=International Standard Book Number
ISDN=Integrated Services Digital Network
ISO=International Organization for Standardization; also short for a image of an ISO9660 (CD-ROM) file system
ISSN=International Standard Serial Number
KDE=K-Desktop Environment; Kolorful Diskfilling Environment
KISS=Keep it simple, stupid
LIFO=Last In, First Out
LUG=Linux User Group
MCSE=Minesweeper Consultant and Solitaire Expert (User Friendly)
MMS=Multimedia Messaging Service
MMX=Multimedia Extension
MP3=MPEG (Moving Picture Experts Group) 1 Audio Layer 3
MPEG=Moving Picture Experts Group
MSIE=Microsoft Internet Explorer
NSFW=Not Safe For Work
OOP=Object-Oriented Programming
OS=Operating System; Open Source
OSI=Open Source Initiative; Open Systems Interconnection
OSS=Open Source Software
PHP[2345]?=Programmers Hate PHP ;-)
QA=Quality Assurance
RAM=Random Access Memory
ROM=Read Only Memory
SMD=Surface Mounted Devices
SMS=Short Message Service
SMTP=Simple Mail Transfer Protocol
SPF=Sender Policy Framework, formerly Sender Permitted From
SSI=Server-Side Includes
TLA=Three Letter Acronym
UI=User Interface
UMTS=Universal Mobile Telecommunications System
URI=Uniform Resource Indicator
URL=Uniform Resource Locator
USB=Universal Serial Bus
VM=Virtual Machine
VoIP=Voice over IP
WYSIWYG=What you see is what you get
XHTML=Extensible Hypertext Markup Language
XML=Extensible Markup Language
""".strip()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

from docutils import nodes
from docutils.parsers.rst import directives, Directive

from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, TextLexer

match = ['code-block', 'sourcecode', 'pygments']


class Pygments(Directive):
    """ Source code syntax hightlighting using Pygments.

    Usage example::

        .. sourcecode:: python
            :linenos:

            #!/usr/bin/env python
            print "Hello World!

    ``linenos`` enables line numbering. A pygment CSS is not provided by
    acrylamid but can easily get one by using the ``pygmentize`` script
    shipped with Pygments.

    ::

        $> pygmentize -S trac -f html > pygments.css

    To get a list of all available styles use the interactive python interpreter.

    ::

        >>> from pygments import styles
        >>> print list(styles.get_all_styles())
    """
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = True
    option_spec = {
        'linenos': directives.flag,
    }

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        formatter = HtmlFormatter(noclasses=False)
        if 'linenos' in self.options:
            formatter.linenos = 2
        parsed = highlight(u'\n'.join(self.content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]


def makeExtension():
    return Pygments



#!/usr/bin/env python
from distutils.core import setup

"""Welcome to Acrylamid's Documentation!
=====================================

Acrylamid is yet another static blog compiler written in python that aims to
be lightweight, fast and producing high quality output. It is licensed under
BSD Style, 2 clauses.

Features
--------

Acrylamid is a mixture of `nanoc <http://nanoc.stoneship.org/>`_, `Pyblosxom
<http://pyblosxom.bluesock.org/>`_ and `Pelican <http://pelican.notmyidea.org/>`_. It
features mainly:

- blog articles, static pages, tags RSS/Atom feeds and an article overview
- theming support (using jinja2_) and support for jinja2 directly in postings
- Markdown_, reStructuredText_ and pandoc_
- MathML, modern web-typography and hyphenation using `&shy;`
- RSS/Atom import, deployment and a handy CLI
- it's very flexible/configurable and fast

.. _jinja2: http://jinja.pocoo.org/
.. _reStructuredText: http://docutils.sourceforge.net/rst.html
.. _Markdown: http://daringfireball.net/projects/markdown/
.. _pandoc: http://johnmacfarlane.net/pandoc/
.. _AsciiMathML: http://www1.chapman.edu/~jipsen/mathml/asciimath.html

Quickstart
----------

::

    easy_install -U acrylamid

It has actually only one dependency, ``jinja2`` but for convenience it also
installs ``markdown`` and ``translitcodec``. In addition it has support for
PyYAML, reStructuredText, syntax highlighting using pygments, asciimathml
to render MathML and finally smartypants for nicer typography.

::

    easy_install -U docutils pygments asciimathml smartypants

Get acrylamid, edit *conf.py* and *layouts/* and compile with:

::

    $> acrylamid init myblog
        create  myblog/conf.py
        ...
    $> cd myblog/
    $> acrylamid compile && acrylamid view
          warn  using mtime from <fileentry f'content/sample entry.txt'>
        create  '/articles/index.html', written to output/articles/index.html
        create  'Die Verwandlung', written to output/2011/die-verwandlung/index.html
        create  '/atom/index.html', written to output/atom/index.html
        create  '/rss/index.html', written to output/rss/index.html
        create  '/', written to output/index.html
       * Running on http://127.0.0.1:8000/

Real World Example?
*******************

I have released all sources from my personal blog:
`/posativ/blog.posativ.org/ <https://github.com/posativ/blog.posativ.org/>`_. Take a
look at my *conf.py* for some inspiration.


Filters
-------

You can apply various filter to a single entry, to a specific view or globally
and Acrylamid resolve it automatically (some filters conflict with others so
you can for example apply *Markdown* as global filter but render some entries
with reStructuredText). Currently supported by acrylamid, see
`docs/filters.rst <http://acrylamid.readthedocs.org/en/latest/filters.html>`_
for detailed information:

- **Markdown**: rendering Markdown (+asciimathml,pygments,built-in extensions)
- **reST**: reStructuredText (+pygments)
- **pandoc**: Pandoc (+Markdown,textitle,rst,...)
- **HTML**: don't render with Markdown, reStructuredText or Pandoc (it's a conflicting filter)

- **typography**: https://code.google.com/p/typogrify/ (and custom modifications)
- **hyphenation**: hyphenate words (len > 10) based on language
- **summarize**: summarizes posts to 200 words

- **head_offset**: decrease headings by offset
- **jinja2**: write jinja2 in your entries (you can also execute system calls therewith)
- **acronyms**: automatically replace acronyms and abbreviations to help unexperienced users


Commands
--------

See :doc:`commands` for a detailed overview.

::

    %> acrylamid --help
    Usage: acrylamid <subcommand> [options] [args]

    Options:
      -q, --quiet    less verbose
      -v, --verbose  more verbose
      -h, --help     show this help message and exit
      --version      print version details

    Commands:
      init           initializes base structure in DIR
      create  (new)  creates a new entry
      compile (co)   compile blog
      view           fire up built-in webserver
      autocompile    automatic compilation and serving (short aco)
      clean   (rm)   remove abandoned files
      import         import content from URL
      deploy         run a given TASK

    All subcommands except `init` require a conf.py file.
"""

setup(
    name='acrylamid',
    version='0.3.2',
    author='posativ',
    author_email='info@posativ.org',
    packages=[
        'acrylamid', 'acrylamid.filters', 'acrylamid.views', 'acrylamid.lib',
        'acrylamid.defaults'],
    scripts=['bin/acrylamid'],
    package_data={
        'acrylamid.filters': ['hyph/*.txt'],
        'acrylamid.defaults': ['misc/*', 'xhtml/*', 'html5/*']},
    url='http://pypi.python.org/pypi/acrylamid/',
    license='BSD revised',
    description='yet another static blog generator',
    long_description=__doc__,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content :: News/Diary",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
    ],
    install_requires=[
        'Jinja2>=2.4',
        'Markdown>=2.0.1',
        'translitcodec>=0.2'
    ],
)



# Copyright 2012 sebix <szebi@gmx.at>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py
# Idea by http://nitens.org/taraborelli/texlogo

from acrylamid.filters import Filter
from acrylamid.errors import AcrylamidException

class Tex(Filter):

    match = ['metalogo']
    version = '1.0.0'

    priority = 70.0

    def transform(self, text, entry, *args):
        replacings = (('LaTeX', '<span class="latex">L<sup>a</sup>T<sub>e</sub>X</span>'),
                        ('XeTeX', '<span class="latex">X<sub>&#398;</sub>T<sub>e</sub>X</span>'),
                        ('TeX', '<span class="latex">T<sub>e</sub>X</span>'))
        for k in replacings:
            text = text.replace(k[0], k[1])
        return text



# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

import sys
import os
import re
import codecs
import tempfile
import subprocess
import hashlib
import zlib

from fnmatch import fnmatch
from datetime import datetime
from os.path import join, exists, dirname, getmtime, basename
from collections import defaultdict

from acrylamid import log
from acrylamid.errors import AcrylamidException

import traceback
from jinja2 import FileSystemLoader, meta

try:
    import cPickle as pickle
except ImportError:
    import pickle

_slug_re = re.compile(r'[\t !"#$%&\'()*\-/<=>?@\[\\\]^_`{|},.:]+')
_tracked_files = set([])

try:
    import translitcodec
except ImportError:
    from unicodedata import normalize
    translitcodec = None

try:
    import yaml
except ImportError:
    yaml = None


# Borrowed from werkzeug._internal
class _Missing(object):

    def __repr__(self):
        return 'no value'

    def __reduce__(self):
        return '_missing'


# Borrowed from werkzeug.utils
class cached_property(object):
    """A decorator that converts a function into a lazy property. The
    function wrapped is called the first time to retrieve the result
    and then that calculated result is used the next time you access
    the value::

    class Foo(object):

    @cached_property
    def foo(self):
    # calculate something important here
    return 42

    The class has to have a `__dict__` in order for this property to
    work.
    """

    # implementation detail: this property is implemented as non-data
    # descriptor. non-data descriptors are only invoked if there is
    # no entry with the same name in the instance's __dict__.
    # this allows us to completely get rid of the access function call
    # overhead. If one choses to invoke __get__ by hand the property
    # will still work as expected because the lookup logic is replicated
    # in __get__ for manual invocation.

    def __init__(self, func, name=None, doc=None):
        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func
        self._missing = _Missing()

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        value = obj.__dict__.get(self.__name__, self._missing)
        if value is self._missing:
            value = self.func(obj)
            obj.__dict__[self.__name__] = value
        return value


def read(filename, encoding, remap={}):
    """Open filename and read content using specified encoding.  It will try
    to parse the YAML header with yaml.load or fallback (if not available) to
    a naïve key-value parser. Returns offset where the real content begins and
    YAML header.

    :param filename: path to an existing text file
    :param encoding: encoding of this file
    :param remap: remap deprecated/false-written YAML keywords
    """

    def distinguish(value):
        if value == '':
            return None
        elif value.isdigit():
            return int(value)
        elif value.lower() in ['true', 'false']:
             return True if value.capitalize() == 'True' else False
        elif value[0] == '[' and value[-1] == ']':
            return list([unicode(x.strip())
                for x in value[1:-1].split(',') if x.strip()])
        else:
            return unicode(value.strip('"').strip("'"))

    head = []
    i = 0

    with codecs.open(filename, 'r', encoding=encoding, errors='replace') as f:
        while True:
            line = f.readline(); i += 1
            if i == 1 and line.startswith('---'):
                pass
            elif i > 1 and not line.startswith('---'):
                head.append(line)
            elif i > 1 and line.startswith('---'):
                break

    if head and yaml:
        try:
            props = yaml.load(''.join(head))
        except yaml.YAMLError as e:
            raise AcrylamidException('YAMLError: %s' % str(e))
        for key, to in remap.iteritems():
            if key in props:
                props[to] = props[key]
                del props[key]
    else:
        props = {}
        for j, line in enumerate(head):
            if line[0] == '#' or not line.strip():
                continue
            try:
                key, value = [x.strip() for x in line.split(':', 1)]
                if key in remap:
                    key = remap[key]
            except ValueError:
                raise AcrylamidException('%s:%i ValueError: %s\n%s' %
                    (filename, j, line.strip('\n'),
                    ("Either your YAML is malformed or our "
                    "naïve parser is to dumb to read it. Revalidate\n"
                    "your YAML or install PyYAML parser: easy_install -U pyyaml")))
            props[key] = distinguish(value)

    return i, props


class Node(dict):
    """This is a root, an edge and a leaf. Stores predecessor and
    count of views using this leaf."""

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self.refs = 1
        self.prev = None


class FilterTree(list):

    def __init__(self, *args, **kwargs):

        # its a list after all ;-)
        super(FilterTree, self).__init__(*args, **kwargs)

        self.root = Node()
        self.views = {None: self}
        self.paths = {None: []}

    def __iter__(self):
        """Iterating over list of filters of given context."""

        raise NotImplemented('XXX get context with some magic')
        res = []
        context = None
        node = self.views[context]

        while node.prev is not None:
            res.append(node.obj)
            node = node.prev

        return reversed(res)

    def add(self, lst, context):
        """This adds a list of filters and stores the context and the
        reference to that path in self.views."""

        node = self.root
        for key in lst:
            if key not in node:
                node[key] = Node()
                node[key].prev = node
                node = node[key]
            else:
                node = node[key]
                node.refs += 1

        self.views[context] = node
        self.paths[context] = lst

    def path(self, context):
        """Return the actual 'path' a view would use."""

        return self.paths[context]

    def iter(self, context):
        """This returns a generator which yields a tuple containing the count
        of views using this filter list and the filter list itself."""

        path, node = self.path(context)[:], self.root
        i, j = 0, self.root[path[0]].refs

        while True:

            ls = []
            for key in path[:]:
                if node[key].refs != j:
                    j = node[key].refs
                    break

                ls.append(key)
                node = node[key]
                path.pop(0)

            if not ls:
                raise StopIteration

            i += 1
            yield i, ls


class FileEntry:
    """This class gets it's data and metadata from the file specified
    by the filename argument.

    During templating, every (cached) property is available as well as additional
    key, value pairs defined in the YAML header. Note that *tag* is automatically
    mapped to *tags*, *filter* to *filters* and *static* to *draft*.

    If you have something like

    ::

        ---
        title: Foo
        image: /path/to/my/image.png
        ---

    it is available in jinja2 templates as entry.image"""

    __keys__ = ['permalink', 'date', 'year', 'month', 'day', 'filters', 'tags',
                'title', 'author', 'content', 'description', 'lang', 'draft',
                'extension', 'slug']

    def __init__(self, filename, conf):
        """parsing FileEntry's YAML header."""

        self.filename = filename
        self.mtime = os.path.getmtime(filename)
        self.props = dict((k, v) for k, v in conf.iteritems()
                        if k in ['author', 'lang', 'encoding', 'date_format',
                                 'permalink_format', 'email'])

        i, props = read(filename, self.props['encoding'],
                        remap={'tag': 'tags', 'filter': 'filters', 'static': 'draft'})
        self.offset = i
        self.props.update(props)

        fx = self.props.get('filters', [])
        if isinstance(fx, basestring):
            fx = [fx]

        self.filters = FilterTree(fx)

    def __repr__(self):
        return "<FileEntry f'%s'>" % self.filename

    @cached_property
    def permalink(self):
        """Actual permanent link, depends on entry's property and ``permalink_format``.
        If you set permalink in the YAML header, we use this as permalink otherwise
        the URL without trailing *index.html.*"""

        try:
            return self.props['permalink']
        except KeyError:
            return expand(self.props['permalink_format'].rstrip('index.html'), self)

    @cached_property
    def date(self):
        """return :class:`datetime.datetime` object.  Either converted from given key
        and ``date_format`` or fallback to modification timestamp of the file."""

        # alternate formats from pelican.utils, thank you!
        # https://github.com/ametaireau/pelican/blob/master/pelican/utils.py
        formats = ['%Y-%m-%d %H:%M', '%Y/%m/%d %H:%M',
                   '%Y-%m-%d', '%Y/%m/%d',
                   '%d-%m-%Y', '%Y-%d-%m', # Weird ones
                   '%d/%m/%Y', '%d.%m.%Y',
                   '%d.%m.%Y %H:%M', '%Y-%m-%d %H:%M:%S']

        if 'date' not in self.props:
            log.warn("using mtime from %r" % self.filename)
            return datetime.fromtimestamp(self.mtime)

        string = re.sub(' +', ' ', self.props['date'])
        formats.insert(0, self.props['date_format'])

        for date_format in formats:
            try:
                return datetime.strptime(string, date_format)
            except ValueError:
                pass
        else:
            raise AcrylamidException("%r is not a valid date" % string)

    @property
    def year(self):
        """entry's year (Integer)"""
        return self.date.year

    @property
    def month(self):
        """entry's month (Integer)"""
        return self.date.month

    @property
    def day(self):
        """entry's day (Integer)"""
        return self.date.day

    @property
    def tags(self):
        """per-post list of applied tags, if any.  If you applied a single string it
        is used as one-item array."""

        fx = self.props.get('tags', [])
        if isinstance(fx, basestring):
            return [fx]
        return fx

    @property
    def title(self):
        """entry's title."""
        return self.props.get('title', 'No Title!')

    @property
    def author(self):
        """entry's author as set in entry or from conf.py if unset"""
        return self.props['author']

    @property
    def email(self):
        """the author's email address"""
        return self.props['email']

    @property
    def draft(self):
        """If set to True, the entry will not appear in articles, index, feed and tag view."""
        return True if self.props.get('draft', False) else False

    @property
    def lang(self):
        return self.props['lang']

    @property
    def extension(self):
        """filename's extension without leading dot"""
        return os.path.splitext(self.filename)[1][1:]

    @property
    def source(self):
        with codecs.open(self.filename, 'r', encoding=self.props['encoding'],
        errors='replace') as f:
            return ''.join(f.readlines()[self.offset:]).strip()

    @property
    def content(self):
        """Returns the processed content.  This one of the core functions of
        acrylamid: it compiles incrementally the filter chain using a tree
        representation and saves final output or intermediates to cache, so
        we can rapidly re-compile the whole content.

        The cache is rather dumb: acrylamid can not determine wether it's
        abandoned or differs only in a single character. Thus, to minimize
        the overhead the content is zlib-compressed."""

        # previous value
        pv = None

        # this is our cache filename
        obj = md5(self.filename)

        for i, fxs in self.filters.iter(context=self.context):
            key = md5(i, fxs)

            try:
                rv = cache.get(obj, key, mtime=self.mtime)
                if rv is None:
                    res = self.source if pv is None else pv
                    for f in fxs:
                        res = f.transform(res, self, *f.args)
                    pv = cache.set(obj, key, data=res)
                    self.has_changed = True
                else:
                    pv = rv
            except (IndexError, AttributeError):
                # jinja2 will ignore these Exceptions, better to catch them before
                traceback.print_exc(file=sys.stdout)

        return pv

    @property
    def slug(self):
        """ascii safe entry title"""
        return safeslug(self.title)

    @property
    def description(self):
        """first 50 characters from the source"""
        # XXX this is really poor
        return self.source[:50].strip() + '...'

    @property
    def md5(self):
        return md5(self.filename, self.title, self.date)

    @property
    def has_changed(self):
        """Check wether the entry has changed using the following conditions:

        - cache file does not exist -> has changed
        - cache file does not contain required filter intermediate -> has changed
        - entry's file is newer than the cache's one -> has changed
        - otherwise -> not changed
        """

        path = md5(self.filename)
        filters = self.filters.iter(self.context)

        for i, fxs in filters:
            if not cache.has_key(path, md5(i, fxs)):
                return True
        else:
            return getmtime(self.filename) > cache.get_mtime(path)

    def keys(self):
        return list(iter(self))

    def __iter__(self):
        for k in self.__keys__:
            yield k

    def __getitem__(self, key):
        if key in self.__keys__:
            return getattr(self, key)
        return self.props[key]


class ExtendedFileSystemLoader(FileSystemLoader):

    # Acrylamid views (should) process templates on the fly thus we
    # don't have a 1. "select template", 2. "render template" stage
    resolved = {}

    def load(self, environment, name, globals=None):
        """patched `load` to add a has_changed property providing information
        whether the template or its parents have changed."""

        def resolve(parent):
            """We check whether any dependency (extend-block) has changed and
            update the bucket -- recursively. Returns True if the template
            itself or any parent template has changed. Otherwise False."""

            if parent in self.resolved:
                return self.resolved[parent]

            source, filename, uptodate = self.get_source(environment, parent)
            bucket = bcc.get_bucket(environment, parent, filename, source)
            p = bcc._get_cache_filename(bucket)
            has_changed = getmtime(filename) > getmtime(p) if exists(p) else True

            if has_changed:
                # updating cached template if timestamp has changed
                code = environment.compile(source, parent, filename)
                bucket.code = code
                bcc.set_bucket(bucket)

                self.resolved[parent] = True
                return True

            ast = environment.parse(source)
            for name in meta.find_referenced_templates(ast):
                rv = resolve(name)
                if rv:
                    # XXX double-return to break this recursion?
                    return True

        if globals is None:
            globals = {}

        source, filename, uptodate = self.get_source(environment, name)

        bcc = environment.bytecode_cache
        bucket = bcc.get_bucket(environment, name, filename, source)
        has_changed = bool(resolve(name))

        code = bucket.code
        if code is None:
            code = environment.compile(source, name, filename)

        tt = environment.template_class.from_code(environment, code, globals, uptodate)
        tt.has_changed = has_changed
        return tt


def filelist(content_dir, entries_ignore=[]):
    """Gathering all entries in content_dir except entries_ignore via fnmatch."""

    flist = []
    for root, dirs, files in os.walk(content_dir):
        for f in files:
            if f[0] == '.':
                continue
            path = join(root, f)
            fn = filter(lambda p: fnmatch(path, os.path.join(content_dir, p)), entries_ignore)
            if not fn:
                flist.append(path)
    return flist


def union(*args, **kwargs):
    """Takes a list of dictionaries and performs union of each.  Can take additional
    key=values as parameters which may overwrite or add a key/value-pair."""

    d = args[0]
    for dikt in args[1:]:
        d.update(dict)

    d.update(kwargs)
    return d


def mkfile(content, path, ctime=0.0, force=False, dryrun=False, **kwargs):
    """Creates entry in filesystem. Overwrite only if content differs.

    :param content: rendered html/xml to write
    :param path: path to write to
    :param ctime: time needed to compile
    :param force: force overwrite, even nothing has changed (defaults to `False`)
    :param dryrun: don't write anything."""

    # XXX use hashing for comparison
    if exists(dirname(path)) and exists(path):
        with file(path) as f:
            old = f.read()
        if content == old and not force:
            event.identical(path)
        else:
            if not dryrun:
                with open(path, 'w') as f:
                    f.write(content)
            event.changed(path=path, ctime=ctime)
    else:
        try:
            if not dryrun:
                os.makedirs(dirname(path))
        except OSError:
            # dir already exists (mostly)
            pass
        if not dryrun:
            with open(path, 'w') as f:
                f.write(content)
        event.create(path=path, ctime=ctime)


def md5(*objs,  **kw):
    """A multifunctional hash function which can take one or multiple objects
    and you can specify which character-sequences you want calculate to a MD5."""

    attr = kw.get('attr', lambda o: o.__str__())  # positional arguments before *args issue
    h = hashlib.md5()
    for obj in objs:
        h.update(attr(obj))

    return h.hexdigest()


def expand(url, obj):
    """expanding '/:year/:slug/' scheme into e.g. '/2011/awesome-title/"""

    for k in obj:
        if not k.endswith('/') and (':' + k) in url:
            url = url.replace(':'+k, str(obj[k]))
    return url


def joinurl(*args):
    """joins multiple urls to one single domain without loosing root (first element)"""

    r = []
    for i, mem in enumerate(args):
        if i > 0:
            mem = str(mem).lstrip('/')
        r.append(mem)
    return join(*r)


def safeslug(slug):
    """Generates an ASCII-only slug.  Borrowed from
    http://flask.pocoo.org/snippets/5/"""

    result = []
    if translitcodec:
        slug = slug.encode('translit/long').strip()
    for word in _slug_re.split(slug.lower()):
        if not translitcodec:
            word = normalize('NFKD', word).encode('ascii', 'ignore').strip()
            log.once(warn="no 'translitcodec' found, using NFKD algorithm")
        if word and not word[0] in '-:':
            result.append(word)
    return unicode('-'.join(result))


def paginate(list, ipp, func=lambda x: x, salt=None, orphans=0):
    """Yields a triple (index, list of entries, has changed) of a paginated
    entrylist.  It will first filter by the specified function, then split the
    ist into several sublists and check wether the list or an entry has changed.

    :param list: the entrylist containing FileEntry instances.
    :param ipp: items per page
    :param func: filter list of entries by this function
    :param salt: uses as additional identifier in memoize
    :param orphans: avoid N orphans on last page.
    """

    # apply filter function and prepare pagination with ipp
    res = filter(func, list)
    res = [res[x*ipp:(x+1)*ipp] for x in range(len(res)/ipp+1)
           if res[x*ipp:(x+1)*ipp]]

    if len(res) >= 2 and len(res[-1]) <= orphans:
        res[-2].extend(res[-1])
        res.pop(-1)

    j = len(res)
    for i, entries in enumerate(res):

        i += 1
        next = None if i == 1 else i-1
        curr = i
        prev = None if i >= j else i+1

        # get caller, so we can set a unique and meaningful hash-key
        frame = log.findCaller()
        if salt is None:
            hkey = '%s:%s-hash-%i' % (basename(frame[0]), frame[2], i)
        else:
            hkey = '%s:%s-hash-%s-%i' % (basename(frame[0]), frame[2], salt, i)

        # calculating hash value and retrieve memoized value
        hv = md5(*entries, attr=lambda o: o.md5)
        rv = cache.memoize(hkey)

        if rv == hv:
            # check if a FileEntry-instance has changed
            if bool(filter(lambda e: e.has_changed, entries)):
                has_changed = True
            else:
                has_changed = False
        else:
            # save new value for next run
            cache.memoize(hkey, hv)
            has_changed = True

        yield (next, curr, prev), entries, has_changed


def escape(string):
    """Escape string to fit to the YAML standard.  I did not read the
    specs, though."""

    if filter(lambda c: c in string, '\'\"#:'):
        if '"' in string:
            return '\'' + string + '\''
        else:
            return '\"' + string + '\"'
    return string


def system(cmd, stdin=None, **kw):
    """A simple front-end to python's horrible Popen-interface which lets you
    run a single shell command (only one, semicolon and && is not supported by
    os.execvp(). Does not catch OSError!

    :param cmd: command to run (a single string or a list of strings).
    :param stdin: optional string to pass to stdin.
    :param kw: everything there is passed to Popen.
    """

    try:
        if stdin:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                 stdin=subprocess.PIPE, **kw)
            result, err = p.communicate(stdin)
        else:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kw)
            result, err = p.communicate()

    except OSError as e:
        raise OSError(e.strerror)

    retcode = p.poll()
    if err or retcode != 0:
        if not err.strip():
            err = 'process exited with %i.' % retcode
        raise AcrylamidException(err.strip())
    return result.strip()


class cache(object):
    """A cache that stores the entries on the file system.  Borrowed from
    werkzeug.contrib.cache, see their AUTHORS and LICENSE for additional
    copyright information.

    cache is designed as singleton and should not constructed using __init__ .
    >>> cache.init('.mycache/')
    >>> cache.get(key, default=None, mtime=0.0)
    >>> cache.set(key, value)"""

    _fs_transaction_suffix = '.__ac_cache'
    cache_dir = '.cache/'
    tracked = defaultdict(set)
    mode = 0600

    @classmethod
    def _get_filename(self, hash):
        return join(self.cache_dir, hash)

    @classmethod
    def _list_dir(self):
        """return a list of (fully qualified) cache filenames"""
        return [join(self.cache_dir, fn) for fn in os.listdir(self.cache_dir)
                if not fn.endswith('.cache')]

    @classmethod
    def init(self, cache_dir=None, mode=0600):
        """
        :param cache_dir: the directory where cache files are stored.
        :param mode: the file mode wanted for the cache files, default 0600
        """
        if cache_dir:
            self.cache_dir = cache_dir
        if mode:
            self.mode = mode
        if not exists(self.cache_dir):
            try:
                os.mkdir(self.cache_dir, 0700)
            except OSError:
                log.fatal("could not create directory '%s'" % self.cache_dir)
                sys.exit(1)

    @classmethod
    def clear(self):
        for fname in self._list_dir():
            try:
                os.remove(fname)
            except (IOError, OSError):
                pass

    @classmethod
    def clean(self, dryrun=False):
        """Remove abandoned cache files that are not accessed during a compilation.
        This does not affect jinja2 templates or cache's memoize file *.cache/info*.

        This does also remove abandoned intermediates from a cache file (they accumulate
        over time).

        :param dryrun: don't remove files
        """
        # first we search for cache files from entries that have vanished
        for path in set(self._list_dir()).difference(set(self.tracked.keys())):
            if not dryrun:
                os.remove(path)

        # next we clean the cache files itself
        for path, keys in self.tracked.iteritems():

            try:
                with open(path, 'rb') as fp:
                    obj = pickle.load(fp)
                    found = set(obj.keys())
            except (OSError, IOError, pickle.PickleError):
                obj, found = {}, set([])

            try:
                for key in found.difference(set(keys)):
                    obj.pop(key)
                with open(path, 'wb') as fp:
                    pickle.dump(obj, fp, pickle.HIGHEST_PROTOCOL)
            except (OSError, IOError, pickle.PickleError):
                pass

    @classmethod
    def has_key(self, obj, key):
        """check wether cache file has key and track them as used (= not abandoned)."""
        filename = self._get_filename(obj)
        self.tracked[filename].add(key)

        try:
            with open(filename, 'rb') as fp:
                return key in pickle.load(fp)
        except (OSError, IOError, pickle.PickleError):
            return False

    @classmethod
    def get(self, obj, key, default=None, mtime=0.0):
        """Restore data from obj[key] if mtime has not changed or return default.

        :param obj: object's hash that will be used as filename
        :param key: key of this data
        :param default: default return value
        :param mtime: modification timestamp as float value
        """
        try:
            filename = self._get_filename(obj)
            if mtime > getmtime(filename):
                os.remove(filename)
                return default
            with open(filename, 'rb') as fp:
                return zlib.decompress(pickle.load(fp)[key])
            os.remove(filename)
        except (OSError, IOError, KeyError, pickle.PickleError, zlib.error):
            pass
        return default

    @classmethod
    def set(self, obj, key, data):
        """Save a key, value pair into a blob using pickle and moderate zlib
        compression (level 6). We simply save a dictionary containing all
        different intermediates (from every view) of an entry.

        :param obj: the object's hash, used as filename
        :param key: dictionary key where we store the data
        :param data: a string we compress with zlib and afterwards save
        """
        filename = self._get_filename(obj)

        if exists(filename):
            try:
                with open(filename, 'rb') as fp:
                    rv = pickle.load(fp)
            except pickle.PickleError:
                rv = {}
            try:
                with open(filename, 'wb') as fp:
                    rv[key] = zlib.compress(data, 6)
                    pickle.dump(rv, fp, pickle.HIGHEST_PROTOCOL)
            except (IOError, OSError):
                pass
        else:
            try:
                fd, tmp = tempfile.mkstemp(suffix=self._fs_transaction_suffix,
                                           dir=self.cache_dir)
                with os.fdopen(fd, 'wb') as fp:
                    pickle.dump({key: zlib.compress(data, 6)}, fp, pickle.HIGHEST_PROTOCOL)
                os.rename(tmp, filename)
                os.chmod(filename, self.mode)
            except (IOError, OSError):
                pass

        return data

    @classmethod
    def get_mtime(self, key, default=0.0):
        filename = self._get_filename(key)
        try:
            mtime = getmtime(filename)
        except (OSError, IOError):
            return default
        return mtime

    @classmethod
    def memoize(self, key, value=None):
        """Memoize (stores) key/value pairs into a single file in
        `cache_dir`/info."""

        filename = join(self.cache_dir, 'info')
        self.tracked[filename].add(key)

        if not exists(filename):
            try:
                fd, tmp = tempfile.mkstemp(suffix=self._fs_transaction_suffix,
                                           dir=self.cache_dir)
                with os.fdopen(fd, 'wb') as fp:
                    pickle.dump({}, fp, pickle.HIGHEST_PROTOCOL)
                os.rename(tmp, filename)
                os.chmod(filename, self.mode)
            except (IOError, OSError) as e:
                raise AcrylamidException(str(e.message))

        if not isinstance(key, basestring):
            raise TypeError('key must be a string')

        if value is not None:
            with open(filename, 'rb') as fp:
                values = pickle.load(fp)
            values[key] = value
            with open(filename, 'wb') as fp:
                pickle.dump(values, fp, pickle.HIGHEST_PROTOCOL)
        else:
            with open(filename, 'rb') as fp:
                return pickle.load(fp).get(key, None)


def track(f):
    """decorator to track files when event.create|change|skip is called."""
    def dec(cls, path, *args, **kwargs):
        global _tracked_files
        _tracked_files.add(path)
        return f(cls, path, *args, **kwargs)
    return dec


def get_tracked_files():
    global _tracked_files
    return _tracked_files


class event:
    """this helper class provides an easy mechanism to give user feedback of
    created, changed or deleted files.  As side-effect every non-destructive
    call will add the given path to the global tracking list and makes it
    possible to remove unused files (e.g. after you've changed your permalink
    scheme or just reworded your title).

    This class is a singleton and should not be initialized."""

    @classmethod
    def __init__(self):
        raise NotImplemented

    @classmethod
    @track
    def create(self, path, ctime=None):
        if ctime:
            log.info("create  [%.2fs] %s", ctime, path)
        else:
            log.info("create  %s", path)

    @classmethod
    @track
    def changed(self, path, ctime=None):
        if ctime:
            log.info("update  [%.2fs] %s", ctime, path)
        else:
            log.info("update  %s", path)

    @classmethod
    @track
    def skip(self, path):
        log.skip("skip  %s", path)

    @classmethod
    @track
    def identical(self, path):
        log.skip("identical  %s", path)

    @classmethod
    def removed(self, path):
        log.info("removed  %s", path)



# -*- encoding: utf-8 -*-
#
# Copyright 2012 posativ <info@posativ.org>. All rights reserved.
# License: BSD Style, 2 clauses. see acrylamid/__init__.py

from HTMLParser import HTMLParser as SystemsParser, HTMLParseError
from cgi import escape


def format(attrs):
    res = []
    for key, value in attrs:
        if value is None:
            res.append(key)
        else:
            res.append('%s="%s"' % (key, escape(value)))
    return ' '.join(res)


class HTMLParser(object, SystemsParser):
    """A more useful base HTMLParser.  This class contains the processed but untouched
    result in self.result. It is intended to use this class to avoid HTML mess up."""

    def __init__(self, html):
        SystemsParser.__init__(self)
        self.result = []
        self.stack = []

        self.feed(html)

    def handle_starttag(self, tag, attrs):

        self.stack.append(tag)
        self.result.append('<%s %s>' % (tag, format(attrs)) if attrs else '<%s>' % tag)

    def handle_data(self, data):
        self.result.append(data)

    def handle_endtag(self, tag):
        try:
            self.stack.pop()
        except IndexError:
            pass
        self.result.append('</%s>' % tag)

    def handle_startendtag(self, tag, attrs):
        self.result.append('<%s %s/>' % (tag, format(attrs)))

    def handle_entityref(self, name):
        self.result.append('&' + name + ';')

    def handle_charref(self, char):
        self.result.append('&#' + char + ';')


__all__ = ['HTMLParser', 'HTMLParseError']
